"""Global collection of auxiliary methods."""

import ipaddress
import math
import os
from typing import Any
from urllib.parse import urlparse

import phonenumbers
from bson.objectid import ObjectId
from email_validator import EmailNotValidError, validate_email

from .globals import REGEX


def is_password(password: str | None) -> bool:
    """Validate Password."""
    if not REGEX["password"].match(str(password)):
        return False
    return True


def to_human_size(size: int) -> str:
    """Convert number of bytes to readable format."""
    idx = int(math.floor(math.log(size) / math.log(1024)))
    size = size if size < 1024 else abs(round(size / pow(1024, idx), 2))
    order = ["bytes", "KB", "MB", "GB", "TB"][idx]
    return f"{size} {order}"


def get_file_size(path: str) -> int:
    """Get file size in bytes."""
    size: int = os.path.getsize(path)
    return size


def normal_email(email: str | None) -> str | None:
    """Normalizing email address.
    
    Use this before requeste to a database.
    For example, on the login page.
    """
    normal: str | None = None
    try:
        emailinfo = validate_email(str(email), check_deliverability=False)
        normal = emailinfo.normalized
    except EmailNotValidError:
        pass
    return normal


def is_email(email: str | None) -> bool:
    """Validate Email address."""
    try:
        validate_email(str(email), check_deliverability=True)
    except EmailNotValidError:
        return False
    return True


def is_url(url: str | None) -> bool:
    """Validate URL address."""
    result = urlparse(str(url))
    if not result.scheme or not result.netloc:
        return False
    return True


def is_ip(address: str | int | None) -> bool:
    """Validate IP address."""
    try:
        ipaddress.ip_address(str(address))
    except ValueError:
        return False
    return True


def is_color(color_code: str | None) -> bool:
    """Validate Color code."""
    if REGEX["color_code"].match(str(color_code)) is None:
        return False
    return True


def is_phone(number: str | None) -> bool:
    """Validate Phone number."""
    try:
        phone = phonenumbers.parse(str(number))
        if not phonenumbers.is_valid_number(phone):
            return False
    except phonenumbers.phonenumberutil.NumberParseException:
        return False
    return True


def is_mongo_id(oid: Any) -> bool:
    """Validation of the Mongodb identifier."""
    return ObjectId.is_valid(oid)


def hash_to_obj_id(hash: str | None) -> ObjectId | None:
    """Get ObjectId from hash string."""
    return ObjectId(hash) if bool(hash) else None
